@@ -1,3 +1,4 @@
+import asyncio
 from .structures import EventHandler, EventCall
 
 class EventSpace:
@@ -9,7 +10,7 @@ class EventSpace:
     
     def register_handler(self, handler):
         if handler.event not in self.events:
-            self.events[handler.event] = {}
+            self.events[handler.event]: dict[str, dict[int, list[EventHandler]]] = {}
         
         # Check if the priority is already registered.
         if handler.priority not in self.events[handler.event]:
@@ -47,7 +48,7 @@ class EventSpace:
 
 
     # Define the "on" method
-    def on(self, event_name: str, priority: int=0, register_ra: bool=True, function: bool=None, requirement_exceptions: bool=False, is_async: bool=False, **kwargs):
+    def on(self, event_name: str, priority: int=0, register_ra: bool=True, function: bool=None, requirement_exceptions: bool=False, is_async: bool=None, **kwargs):
         """Attaches a handler
 
         Args:
@@ -75,6 +76,36 @@ class EventSpace:
         wrapper(function)
 
 
+    async def async_call(self, event_name: str, *args, **kwargs):
+        """Calls an event with asynchronous handlers
+
+        Args:
+            event_name (str): Name of the event being called.
+
+        Returns:
+            EventCall
+        """
+        # Define the event call object.
+        event_call = EventCall(event_name=event_name)
+ 
+        # Return if there isn't an event registered with this name.
+        if event_name not in self.events:
+            return event_call
+
+        # Run the handlers.
+        calls = []
+        for handler in self.get_handlers(event_name):
+            handler: EventHandler
+            handler_return = handler.async_call(event_call, args=args, kwargs=kwargs)
+            if handler.is_async:
+                calls.append(handler_return)
+            else:
+                event_call.responses.append(handler_return)
+        event_call.responses = await asyncio.gather(*calls)
+        # Return the event
+        return event_call
+
+
     # Define the "call" method
     def call(self, event_name: str, cancellable: bool=False, blocking: bool=True, first_response: bool=False, *args, **kwargs):
         """Calls an event
@@ -98,17 +129,17 @@ class EventSpace:
 
         # Run the handlers.
         for handler in self.get_handlers(event_name):
+            handler: EventHandler
             # Run the handler
             response = handler.call(event_call, args=args, kwargs=kwargs)
             event_call.responses.append(response)
             if response != None:
-                event_call.response = response
                 # If this is the first response, break the loop
                 if first_response:
                     break
 
             # If the event was cancelled, do not run the next handlers.
-            if event_call.cancelled and blocking:
+            if (event_call.cancelled and blocking) or event_call.stopped:
                 break
 
         # Return the event
