@@ -6,7 +6,7 @@ from .exceptions import UncancellableEvent, UnfulfilledRequirement
 
 # Wrapper for event handlers
 class EventHandler:
-    def __init__(self, function, event: str, priority: int, event_space, additional_kwargs: dict = {}, execution_requirements: dict = {}, requirement_exceptions: bool = False, is_async: bool = False):
+    def __init__(self, function, event: str, priority: int, event_space, additional_kwargs: dict = {}, execution_requirements: dict = {}, requirement_exceptions: bool = False, is_async: bool = None):
         self.function = function
         self.event = event
         self.priority = priority
@@ -14,7 +14,7 @@ class EventHandler:
         self.event_space = event_space
         self.execution_requirements = execution_requirements
         self.requirement_exceptions = requirement_exceptions
-        self.is_async = is_async
+        self.is_async = is_async if isinstance(is_async, bool) else inspect.iscoroutinefunction(function)
 
     @property
     def origin_module(self):
@@ -31,6 +31,16 @@ class EventHandler:
     def registered(self):
         return self.event_space.handler_registered(self)
 
+    async def async_call(self, call, args, kwargs):
+        kwargs = kwargs | self.additional_kwargs
+        for argument in self.execution_requirements:
+            if argument not in kwargs or self.execution_requirements[argument] != kwargs[argument]:
+                if self.requirement_exceptions:
+                    raise UnfulfilledRequirement(f"Argument {argument} was unmet.")
+                return
+        return await self.function(event=call, *args, **kwargs)
+
+
     def call(self, call, args, kwargs):        
         kwargs = kwargs | self.additional_kwargs
         for argument in self.execution_requirements:
@@ -65,8 +75,9 @@ class EventCall:
     def __init__(self, event_name, cancellable=False):
         self.__name = event_name
         self.__cancelled = False
+        self.__stopped = False
+        self.__response = None
         self.cancellable = cancellable        
-        self.response = None
         self.responses = []
     
     # Cancelling is only one-way. 
@@ -76,11 +87,22 @@ class EventCall:
             return
         raise UncancellableEvent(f'{self.event_name} is not an cancellable event.')
     
+    def stop(self):
+        self.__stopped = True
+
     # Using it as a property since we don't want it to be writable 
     @property
     def cancelled(self):
         return self.__cancelled
 
+    @property
+    def response(self):
+        return next((i for i in self.responses if i), None)
+
+    @property
+    def stopped(self):
+        return self.__stopped
+
     @property
     def name(self):
         return self.__name
